@@ -12,9 +12,10 @@
 # limitations under the License.
 
 import argparse
+import io
+import json
 import re
 import sys
-import unittest
 from unittest import mock
 
 import ddt
@@ -24,7 +25,6 @@ from keystoneauth1.exceptions import DiscoveryFailure
 from keystoneauth1.identity.generic.password import Password as ks_password
 from keystoneauth1 import session
 import requests_mock
-from six import moves
 from testtools import matchers
 
 import cinderclient
@@ -35,6 +35,7 @@ from cinderclient import shell
 from cinderclient.tests.unit import fake_actions_module
 from cinderclient.tests.unit.fixture_data import keystone_client
 from cinderclient.tests.unit import utils
+from cinderclient.tests.unit.v3 import fakes
 
 
 @ddt.ddt
@@ -64,7 +65,7 @@ class ShellTest(utils.TestCase):
     def shell(self, argstr):
         orig = sys.stdout
         try:
-            sys.stdout = moves.StringIO()
+            sys.stdout = io.StringIO()
             _shell = shell.OpenStackCinderShell()
             _shell.main(argstr.split())
         except SystemExit:
@@ -119,9 +120,9 @@ class ShellTest(utils.TestCase):
         # Some expected help output, including microversioned commands
         required = [
             r'.*?^usage: ',
-            r'.*?(?m)^\s+create\s+Creates a volume.',
-            r'.*?(?m)^\s+summary\s+Get volumes summary.',
-            r'.*?(?m)^Run "cinder help SUBCOMMAND" for help on a subcommand.',
+            r'.*?^\s+create\s+Creates a volume.',
+            r'.*?^\s+summary\s+Get volumes summary.',
+            r'.*?^Run "cinder help SUBCOMMAND" for help on a subcommand.',
         ]
         help_text = self.shell('help')
         for r in required:
@@ -131,7 +132,7 @@ class ShellTest(utils.TestCase):
     def test_help_on_subcommand(self):
         required = [
             r'.*?^usage: cinder list',
-            r'.*?(?m)^Lists all volumes.',
+            r'.*?^Lists all volumes.',
         ]
         help_text = self.shell('help list')
         for r in required:
@@ -141,13 +142,25 @@ class ShellTest(utils.TestCase):
     def test_help_on_subcommand_mv(self):
         required = [
             r'.*?^usage: cinder summary',
-            r'.*?(?m)^Get volumes summary.',
+            r'.*?^Get volumes summary.',
         ]
         help_text = self.shell('help summary')
         for r in required:
             self.assertThat(help_text,
                             matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))
 
+    def test_help_arg_no_subcommand(self):
+        required = [
+            r'.*?^usage: ',
+            r'.*?^\s+create\s+Creates a volume.',
+            r'.*?^\s+summary\s+Get volumes summary.',
+            r'.*?^Run "cinder help SUBCOMMAND" for help on a subcommand.',
+        ]
+        help_text = self.shell('--os-volume-api-version 3.40')
+        for r in required:
+            self.assertThat(help_text,
+                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))
+
     @ddt.data('backup-create --help', '--help backup-create')
     def test_dash_dash_help_on_subcommand(self, cmd):
         required = ['.*?^Creates a volume backup.']
@@ -193,8 +206,13 @@ class ShellTest(utils.TestCase):
         os_auth_url = "http://multiple.service.names/v2.0"
         mocker.register_uri('POST', os_auth_url + "/tokens",
                             text=keystone_client.keystone_request_callback)
+        # microversion support requires us to make a versions request
+        # to the endpoint to see exactly what is supported by the server
         mocker.register_uri('GET',
-                            "http://cinder%i.api.com/v2/volumes/detail"
+                            "http://cinder%i.api.com/"
+                            % count, text=json.dumps(fakes.fake_request_get()))
+        mocker.register_uri('GET',
+                            "http://cinder%i.api.com/v3/volumes/detail"
                             % count, text='{"volumes": []}')
         self.make_env(include={'OS_AUTH_URL': os_auth_url,
                                'CINDER_SERVICE_NAME': 'cinder%i' % count})
@@ -207,7 +225,6 @@ class ShellTest(utils.TestCase):
                           _shell.main,
                           ['list', '--name', 'abc', '--filters', 'name=xyz'])
 
-    @unittest.skip("Skip cuz I broke it")
     def test_cinder_service_name(self):
         # Failing with 'No mock address' means we are not
         # choosing the correct endpoint
@@ -215,7 +232,7 @@ class ShellTest(utils.TestCase):
             self.list_volumes_on_service(count)
 
     @mock.patch('keystoneauth1.identity.v2.Password')
-    @mock.patch('keystoneauth1.adapter.Adapter.get_token',
+    @mock.patch('keystoneauth1.adapter.LegacyJsonAdapter.get_token',
                 side_effect=ks_exc.ConnectFailure())
     @mock.patch('keystoneauth1.discover.Discover',
                 side_effect=ks_exc.ConnectFailure())
@@ -236,14 +253,19 @@ class ShellTest(utils.TestCase):
             tenant_name=self.FAKE_ENV['OS_PROJECT_NAME'],
             username=self.FAKE_ENV['OS_USERNAME'])
 
+    @mock.patch('cinderclient.api_versions.discover_version',
+                return_value=api_versions.APIVersion("3.0"))
     @requests_mock.Mocker()
-    def test_noauth_plugin(self, mocker):
-        os_auth_url = "http://example.com/v2"
+    def test_noauth_plugin(self, mock_disco, mocker):
+        # just to prove i'm not crazy about the mock parameter ordering
+        self.assertTrue(requests_mock.mocker.Mocker, type(mocker))
+
+        os_volume_url = "http://example.com/volumes/v3"
         mocker.register_uri('GET',
                             "%s/volumes/detail"
-                            % os_auth_url, text='{"volumes": []}')
+                            % os_volume_url, text='{"volumes": []}')
         _shell = shell.OpenStackCinderShell()
-        args = ['--os-endpoint', os_auth_url,
+        args = ['--os-endpoint', os_volume_url,
                 '--os-auth-type', 'noauth', '--os-user-id',
                 'admin', '--os-project-id', 'admin', 'list']
         _shell.main(args)
@@ -354,7 +376,7 @@ class TestLoadVersionedActions(utils.TestCase):
 
         self.mock_completion()
 
-    def test_load_versioned_actions(self):
+    def test_load_versioned_actions_v3_0(self):
         parser = cinderclient.shell.CinderClientArgumentParser()
         subparsers = parser.add_subparsers(metavar='<subcommand>')
         shell = cinderclient.shell.OpenStackCinderShell()
@@ -366,6 +388,10 @@ class TestLoadVersionedActions(utils.TestCase):
             "fake_action 3.0 to 3.1",
             shell.subcommands['fake-action'].get_default('func')())
 
+    def test_load_versioned_actions_v3_2(self):
+        parser = cinderclient.shell.CinderClientArgumentParser()
+        subparsers = parser.add_subparsers(metavar='<subcommand>')
+        shell = cinderclient.shell.OpenStackCinderShell()
         shell.subcommands = {}
         shell._find_actions(subparsers, fake_actions_module,
                             api_versions.APIVersion("3.2"), False, [])
@@ -411,12 +437,14 @@ class TestLoadVersionedActions(utils.TestCase):
             expected_help = ("help message (Supported by API versions "
                              "%(start)s - %(end)s)") % {
                 'start': '3.0', 'end': '3.3'}
-            expected_desc = ("help message\n\n    "
-                            "This will not show up in help message\n    ")
+            self.assertIn('help message',
+                          mock_add_parser.call_args_list[0][1]['description'])
+            self.assertIn('This will not show up in help message',
+                          mock_add_parser.call_args_list[0][1]['description'])
             mock_add_parser.assert_any_call(
                 'fake-action',
                 help=expected_help,
-                description=expected_desc,
+                description=mock.ANY,
                 add_help=False,
                 formatter_class=cinderclient.shell.OpenStackHelpFormatter)
 
@@ -499,7 +527,7 @@ class TestLoadVersionedActions(utils.TestCase):
 
     @mock.patch.object(cinderclient.shell.CinderClientArgumentParser,
                        'add_argument')
-    def test_load_actions_with_versioned_args(self, mock_add_arg):
+    def test_load_actions_with_versioned_args_v36(self, mock_add_arg):
         parser = cinderclient.shell.CinderClientArgumentParser(add_help=False)
         subparsers = parser.add_subparsers(metavar='<subcommand>')
         shell = cinderclient.shell.OpenStackCinderShell()
@@ -511,8 +539,13 @@ class TestLoadVersionedActions(utils.TestCase):
         self.assertNotIn(mock.call('--foo', help="second foo"),
                          mock_add_arg.call_args_list)
 
-        mock_add_arg.reset_mock()
-
+    @mock.patch.object(cinderclient.shell.CinderClientArgumentParser,
+                       'add_argument')
+    def test_load_actions_with_versioned_args_v39(self, mock_add_arg):
+        parser = cinderclient.shell.CinderClientArgumentParser(add_help=False)
+        subparsers = parser.add_subparsers(metavar='<subcommand>')
+        shell = cinderclient.shell.OpenStackCinderShell()
+        shell.subcommands = {}
         shell._find_actions(subparsers, fake_actions_module,
                             api_versions.APIVersion("3.9"), False, [])
         self.assertNotIn(mock.call('--foo', help="first foo"),
@@ -523,7 +556,7 @@ class TestLoadVersionedActions(utils.TestCase):
 
 class ShellUtilsTest(utils.TestCase):
 
-    @mock.patch.object(cinderclient.utils, 'print_dict')
+    @mock.patch.object(cinderclient.shell_utils, 'print_dict')
     def test_print_volume_image(self, mock_print_dict):
         response = {'os-volume_upload_image': {'name': 'myimg1'}}
         image_resp_tuple = (202, response)
